#include <QApplication>
#include <QPlainTextEdit>
#include <QDebug>
#include <QMouseEvent>

class EventFilter : public QObject {
public:
    EventFilter(QObject *p)
        : QObject(p)
        , m_mouseMove(QCoreApplication::arguments().contains("-m"))
    {}

    bool eventFilter(QObject *, QEvent *);

private:
    bool m_mouseMove;

};

static void debugMouseEvent(QObject *o,  const QMouseEvent *me)
{
    QString s;
    QDebug str(&s);
    str << o;
    if (me->type() == QEvent::MouseButtonPress)
        str << "Press";
    if (me->type() == QEvent::MouseButtonRelease)
        str << "Release";
    if (me->type() == QEvent::MouseMove)
        str << "Move";
    str << "buttons " << me->buttons() << " at  " << me->pos() << me->globalPos();
    qDebug() << s;

}

bool EventFilter::eventFilter(QObject *o, QEvent *e)
{
    const QEvent::Type t = e->type();
    switch (t) {
    case QEvent::Paint:
    case QEvent::Destroy:
    case QEvent::ParentChange:
    case QEvent::ChildAdded:
    case QEvent::ChildRemoved:
    case QEvent::ChildPolished:
    case QEvent::Timer:
    case QEvent::UpdateRequest:
    case QEvent::MetaCall:
    case QEvent::HoverMove:
    case QEvent::DynamicPropertyChange:
    case QEvent::StyleAnimationUpdate:
    case QEvent::DeferredDelete:
        break;
    case QEvent::MouseButtonPress:
    case QEvent::MouseButtonRelease:
        debugMouseEvent(o,  static_cast<const QMouseEvent *>(e));
        break;
    case QEvent::MouseMove:
        if (m_mouseMove)
            debugMouseEvent(o,  static_cast<const QMouseEvent *>(e));
        break;

    default:
        qDebug() << o << e;
        break;
    }
    return false;
}


int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    qDebug() << QT_VERSION_STR << QGuiApplication::platformName();
    if (!QCoreApplication::arguments().contains("-silent"))
        a.installEventFilter(new EventFilter(&a));

    QPlainTextEdit p1;
    p1.setObjectName("p1");
    p1.resize(300, 200);
    p1.setWindowTitle(QLatin1String("Focus ") + QT_VERSION_STR);
    p1.move(200, 200);
    p1.show();
    QPlainTextEdit p2;
    p2.setObjectName("p2");
    p2.setWindowFlags(p2.windowFlags() |  Qt::WindowDoesNotAcceptFocus);
    p2.setWindowTitle(QLatin1String("NonFocus ") + QT_VERSION_STR);
    p2.resize(300, 200);
    p2.move(800, 200);
    p2.show();
    return a.exec();
}
